@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: b9c5f9ade77e3ddd196f3e130741d31080fd3cde $
 */
 
 #ifndef GMP_MACHINE_H
@@ -44,6 +43,9 @@
 /* Define this if you have mpz_bin_ui */
 #undef HAVE_MPZ_BIN_UI
 
+/* Define this if you have mpz_divisible_p */
+#undef HAVE_MPZ_DIVISIBLE_P
+
 /* Define if your cpp supports the ANSI concatenation operator ## */
 #undef HAVE_ANSI_CONCAT
 
