@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 02f15eea6b3a0c172f4adb042a26b0069c5f133f $
 */
 
 /* template for operator layer row function */
@@ -12,13 +11,13 @@ static void LM_FUNC(rgb_group *s,rgb_group *l,rgb_group *d,
 		    int len,double alpha)
 {
   if (da != sa)
-    MEMCPY(da,sa,sizeof(rgb_group)*len); /* always copy alpha channel */
+    memcpy(da,sa,sizeof(rgb_group)*len); /* always copy alpha channel */
 #define da da da /* protect */
    if (alpha==0.0)
    {
 #ifdef LAYER_DUAL
      if (d != s)
-       MEMCPY(d,s,sizeof(rgb_group)*len);
+       memcpy(d,s,sizeof(rgb_group)*len);
 #endif
       return; 
    }
