@@ -1,7 +1,6 @@
 #include "global.h"
 #include "stralloc.h"
 #include "global.h"
-RCSID("$Id: cc7d948387f5e7b6e04b04207d935f0e3644ded0 $");
 #include "pike_macros.h"
 #include "interpret.h"
 #include "program.h"
@@ -11,6 +10,7 @@ RCSID("$Id: cc7d948387f5e7b6e04b04207d935f0e3644ded0 $");
 #include "bignum.h"
 #include "module_support.h"
 #include "fsort.h"
+#include "pike_types.h"
 
 #include "config.h"
 
@@ -139,7 +139,7 @@ void wf_resultset_clear( struct object *o )
   T(o)->d->num_docs = 0;
 }
 
-struct object *wf_resultset_new( )
+struct object *wf_resultset_new(void)
 {
   struct object *o;
   o = clone_object( resultset_program, 0 );
@@ -147,13 +147,13 @@ struct object *wf_resultset_new( )
   return o;
 }
 
-static void init_rs( )
+static void init_rs(struct object *UNUSED(o))
 {
   THIS->d = 0;
   THIS->allocated_size = 0;
 }
 
-static void free_rs()
+static void free_rs(struct object *UNUSED(o))
 {
   THIS->allocated_size = 0;
   if( THIS->d )  free( THIS->d );
@@ -166,18 +166,18 @@ static void free_rs()
 static void f_resultset_create( INT32 args )
 {
   wf_resultset_clear( Pike_fp->current_object );
-  if( args && sp[-1].type == PIKE_T_ARRAY )
+  if( args && TYPEOF(sp[-1]) == PIKE_T_ARRAY )
   {
     int i;
     struct array *d = sp[-1].u.array;
     for(i = 0; i< d->size; i++ )
-      if( d->item[i].type == PIKE_T_ARRAY )
+      if( TYPEOF(d->item[i]) == PIKE_T_ARRAY )
       {
 	LONGEST di, ri;
 	struct array *a = d->item[i].u.array;
 	if( a->size < 2 )
 	  continue;
-	if( a->item[0].type == PIKE_T_OBJECT )
+	if( TYPEOF(a->item[0]) == PIKE_T_OBJECT )
 	{
 	  push_object( a->item[0].u.object );
 	  get_all_args( "create", 1, "%l", &di );
@@ -185,7 +185,7 @@ static void f_resultset_create( INT32 args )
 	}
 	else
 	  di = a->item[0].u.integer;
-	if( a->item[1].type == PIKE_T_OBJECT )
+	if( TYPEOF(a->item[1]) == PIKE_T_OBJECT )
 	{
 	  push_object( a->item[1].u.object );
 	  get_all_args( "create", 1, "%l", &ri );
@@ -198,7 +198,7 @@ static void f_resultset_create( INT32 args )
       else
       {
 	LONGEST ri;
-	if( d->item[i].type == PIKE_T_OBJECT )
+	if( TYPEOF(d->item[i]) == PIKE_T_OBJECT )
 	{
 	  push_object( d->item[i].u.object );
 	  get_all_args( "create", 1, "%l", &ri );
@@ -216,15 +216,22 @@ static void f_resultset_create( INT32 args )
 static void f_resultset_slice( INT32 args );
 
 static void f_resultset_cast( INT32 args )
-/*! @decl string cast( string type )
+/*! @decl array cast( string type )
  *! Only works when type == @expr{"array"@}. Returns the resultset
  *! data as a array.
  */
 {
-  pop_n_elems( args );
-  push_int(0);
-  push_int( 0x7fffffff );
-  f_resultset_slice(2);
+  struct pike_string *type = Pike_sp[-args].u.string;
+  pop_stack(); /* type have at least one more reference. */
+
+  if( type==literal_array_string )
+  {
+    push_int(0);
+    push_int( 0x7fffffff );
+    f_resultset_slice(2);
+  }
+  else
+    push_undefined();
 }
 
 static void f_resultset_memsize( INT32 args )
@@ -287,10 +294,9 @@ static void f_resultset_slice( INT32 args )
   for( i = 0; i<nelems; i++ )
   {
     struct array *a2 = allocate_array(2);
-    res->item[i].type = PIKE_T_ARRAY;
-    res->item[i].u.array = a2;
-    a2->item[0].type = PIKE_T_INT;
-    a2->item[1].type = PIKE_T_INT;
+    SET_SVAL(res->item[i], PIKE_T_ARRAY, 0, array, a2);
+    SET_SVAL_TYPE(a2->item[0], PIKE_T_INT);
+    SET_SVAL_TYPE(a2->item[1], PIKE_T_INT);
     if( THIS->d->hits[i+first].doc_id <= ((((unsigned int)1<<31)-1)) )
 	a2->item[0].u.integer = THIS->d->hits[i+first].doc_id;
     else
@@ -372,7 +378,7 @@ static void f_resultset_dup( INT32 args )
   if(THIS->d)
   {
     ResultSet *d = malloc( THIS->d->num_docs * 8 + 4 );
-    MEMCPY( d, THIS->d, THIS->d->num_docs * 8 + 4 );
+    memcpy( d, THIS->d, THIS->d->num_docs * 8 + 4 );
     T(o)->d = d;
     T(o)->allocated_size = T(o)->d->num_docs;
   }
@@ -428,7 +434,7 @@ static void duplicate_resultset( struct object *dest,
     int size = 4+4*T(src)->allocated_size*2;
     T(dest)->allocated_size = T(src)->allocated_size;
     T(dest)->d              = malloc( size );
-    MEMCPY( (char *)T(dest)->d, (char *)T(src)->d, size );
+    memcpy( T(dest)->d, T(src)->d, size );
   }
 }
 
@@ -884,17 +890,6 @@ static void f_dateset_finalize( INT32 args )
   RETURN_THIS();
 }
 
-static void f_resultset_set( INT32 args )
-{
-  int i;
-  ResultSet *source = THIS->d;
-  if (source) {
-    for( i = 0; i<source->num_docs; i++ )
-      source->hits[i].ranking = 0;
-  }
-  RETURN_THIS();
-}
-
 #define DUP_DATESET()   do {			\
     pop_n_elems(args);				\
     o = dup_dateset();				\
@@ -998,7 +993,7 @@ static void f_resultset_add_many( INT32 args )
   for( i=0;i<a->size;i++ )
   {
     LONGEST ri, di;
-    if( a->item[i].type == PIKE_T_OBJECT )
+    if( TYPEOF(a->item[i]) == PIKE_T_OBJECT )
     {
       push_object( a->item[i].u.object );
       get_all_args( "create", 1, "%l", &di );
@@ -1006,7 +1001,7 @@ static void f_resultset_add_many( INT32 args )
     }
     else
       di = a->item[i].u.integer;
-    if( b->item[i].type == PIKE_T_OBJECT )
+    if( TYPEOF(b->item[i]) == PIKE_T_OBJECT )
     {
       push_object( b->item[i].u.object );
       get_all_args( "create", 1, "%l", &ri );
@@ -1025,7 +1020,7 @@ void init_resultset_program(void)
   start_new_program();
   {  
     ADD_STORAGE( struct result_set_p );
-    add_function("cast", f_resultset_cast, "function(string:mixed)", 0 );
+    add_function("cast", f_resultset_cast, "function(string:mixed)", ID_PRIVATE );
     add_function("create",f_resultset_create,
 		 "function(void|array(int|array(int)):void)",0);
 
@@ -1074,8 +1069,7 @@ void init_resultset_program(void)
   start_new_program();
   {
     struct svalue x;
-    x.type = PIKE_T_PROGRAM;
-    x.u.program = resultset_program;
+    SET_SVAL(x, PIKE_T_PROGRAM, 0, program, resultset_program);
     add_function( "before", f_dateset_before,    "function(int:object)", 0 );
     add_function( "after", f_dateset_after,     "function(int:object)", 0 );
     add_function( "between", f_dateset_between, "function(int,int:object)", 0 );
